//////////////////////////////////////////////////////////////////////////// 
// Image_Processing_toolbox_2
//
// Copyright (c) by S. Parua Biswas, CDAC, Kolkata.  2013
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function [H] = hadamard(n)
    // HADAMARD Hadamard matrix
    // HADAMARD(N) is a Hadamard matrix of order N, that is, a matrix H with elements 1 or -1 such that H'*H = N*EYE(N,N). 
  
    if n<2 then
        error('Input value should be greater than or equal to 2');
    elseif modulo(n,4)~=0 then
        error('n should be integer and after dividing it by 4 the remainder should be zero');
    end
    [f,e] = log2([n n/12 n/20]);
    k = find(f==1/2 & e>0);
    if min(size(n)) > 1 | isempty(k) then
        error('Invalid Input');
    end
    e = e(k)-1;
    
    if k == 1 then   // N = 1 * 2^e
        H = ones();

    elseif k == 2    // N = 12 * 2^e;
       H = [ones(1,12); ones(11,1) toeplitz([-1 -1 1 -1 -1 -1 1 1 1 -1 1],[-1 1 -1 1 1 1 -1 -1 -1 1 -1])];
    elseif k == 3    // N = 20 * 2^e;
       H = [ones(1,20); ones(19,1) hankel([-1 -1 1 1 -1 -1 -1 -1 1 -1 1 -1 1 1 1 1 -1 -1 1],[1 -1 -1 1 1 -1 -1 -1 -1 1 -1 1 -1 1 1 1 1 -1 -1])];
    end
    
    // Kronecker product construction.
    for i = 1:e
        H = [H  H
             H -H];
    end
endfunction
