//////////////////////////////////////////////////////////////////////////// 
// Image_Processing_toolbox_2
//
// Copyright (c) by S. Parua Biswas, CDAC, Kolkata.  2013
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function [H] = hankel(c,r)
    // HANKEL matrix
    // HANKEL(C) is a square Hankel matrix whose first column is c and whose elements are zero below the first anti-diogonal.
    // HANKEL(C,R) is a Hankel matrix whose first column is C and whose last row is R.
    
    c = c(:); //Transpose c matrix
    nc = length(c);
    [c_row c_col]=size(c);
    
    [lhs,rhs]=argn(0);
    if (rhs < 2) then
        r = zeros(c_row,c_col);
    elseif c(nc) ~= r(1)
        disp("hankel:AntiDiagonalConflict")
    end
    r = r(:);
    nr = length(r);
    
    x = [ c; r((2:nr)')]; // build vector of user data
    cidx = (ones():nc)';
    ridx = zeros():(nr-1);
    H = cidx(:,ones(nr,1)) + ridx(ones(nc,1),:);
    H(:) = x(H); 
    
endfunction

