//////////////////////////////////////////////////////////////////////////// 
// Image_Processing_toolbox_2
//
// Copyright (c) by S. Parua Biswas, CDAC, Kolkata.  2013
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function [p,ellipse] = phantom(varargin)
    // PHANTOM Create head phantom image.
    
    [ellipse,n] = parse_inputs(varargin(:));
    
    p = zeros(n,n);
    
    xax = ((0:n-1)-(n-1)/2)/((n-1)/2);
    xg = repmat(xax,n,1);
    
//    disp(size(ellipse,1))
    for k = 1:size(ellipse,1)
        asq = ellipse(k,2)^2;
        bsq = ellipse(k,3)^2;
        
        phi = ellipse(k,6)*%pi/180; //rotation angle in radians
      
        x0 = ellipse(k,4);
        y0 = ellipse(k,5);
        
        A = ellipse(k,1);
        
        x=xg-x0;
        
        y=rot90(xg)-y0;
        
        cosp = cos(phi);
         
        sinp = sin(phi);
        
        idx=find(((x.*cosp + y.*sinp).^2)./asq + ((y.*cosp - x.*sinp).^2)./bsq <= 1);
        idx=idx';
        
        p(idx) = p(idx) +A;
        [m1 n1]=size(idx)

    end
    
    
endfunction

////////////////////////////////////////////////
function [e,n] = parse_inputs(varargin)
    // e is the m-by-6 array which is used for ellipses
    // n is the size of the phantom brain image
    
    [lhs,rhs]=argn(0);
    mfilename = 'parse_inputs';
    n=256; //The default size
    e = [];
    defaults = {'shepp-logan', 'modified shepp-logan'};
    
    for i=1:rhs
        if type(varargin(i))==10 then
            def = mtlb_lower(varargin(i));
            idx = strmatch(def, defaults);
//            disp(def)
//            disp(idx)
            if isempty(idx) then
                eid = sprintf('Images:%s:unknownPhantom',mfilename);
                msg = 'Unknown default phantom selected.';
                error(msg);
            end
            select defaults(idx),
            case 'shepp-logan' then e = shepp_logan(),
            case 'modified shepp-logan' then e = modified_shepp_logan(),
            end
        elseif (NumOfEle(varargin(i))==1) then
            n = varargin(i);
        elseif (ndims(varargin(i))==2 & size(varargin(i),2)==6) then
            e = varargin(i);
        else
            eid = sprintf('Images:%s:invalidInputArgs',mfilename);
            msg = 'Invalid input arguments.';
       //     error(eid,'%s',msg);
            error(msg);
            
        end
    end
//    disp(e)
    if isempty(e) then
        e = modified_shepp_logan();
    end
    
endfunction

///////////////////////////////
// Default head phantom:

function shep=shepp_logan()
    //The default head phantom
    
   //        A      a     b      x0     y0    phi
   //       -------------------------------------- 
    shep = [ 1     .69    .92     0      0     0 
           -.98  .6624  .8740     0  -.0184    0
           -.02  .1100  .3100   .22      0    18
           -.02  .1600  .4100   -.22     0    18
            .01  .2100  .2500    0     .35     0
            .01  .0460  .0460    0      .1     0
            .01  .0460  .0460    0     -.1     0
            .01  .0460  .0230  -.08    -.605   0
            .01  .0230  .0230    0     -.606   0
            .01  .0230  .0460   .06    -.605   0 ];
endfunction

//////////
function toft=modified_shepp_logan()
    // This head phantom is the same as the Shepp-Logan except the intensities 
    // are changed to yield higher contrast in the image. 
     
   //        A      a     b      x0     y0    phi
   //       -------------------------------------- 
    toft = [ 1    .69    .92     0     0     0
           -.8   .6624  .8740    0  -.0184   0
           -.2   .1100  .3100   .22    0    -18
           -.2   .1600  .4100  -.22    0     18
           .1    .2100  .2500    0    .35    0
           .1    .0460  .0460    0    .1     0
           .1    .0460  .0460    0   -.1     0
           .1    .0460  .0230  -.08  -.605   0
           .1    .0230  .0230    0   -.606   0
           .1    .0230  .0460   .06  -.605   0 ];
endfunction

//////////////////////////////
//Count number of element

function y = NumOfEle(x)
    if (type(x)~=1 & type(x)~=10) then
        error('Input array must be of character type');
    end
    [lhs,rhs]=argn(0);
    if rhs~=1 then
        error('Wrong number of input aurguments');        
    end
    [m n]=size(x);
    len_x=length(x(1,:));
       
//      y = 0;
    count_element=0;
//    
    for i = 1:m
        for j = 1:len_x
           count_element=count_element+1;
        end
    end
    y = count_element;
endfunction