#include <math.h>
#include "mex.h"
static void radon(double *pPtr, double *iPtr, double *thetaPtr, int M, int N,int xOrigin, int yOrigin, int numAngles, int rFirst,int rSize);
static char rcs_id[] = "$Revision: 1.10 $";
#define MAXX(x,y) ((x) > (y) ? (x) : (y))
#define I      (prhs[0])
#define THETA  (prhs[1])
#define P      (plhs[0])
#define R      (plhs[1])
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  int numAngles; 
  double *thetaPtr; 
  double *pr1, *pr2;  
  double deg2rad;  
  double temp;  
 int k;  
 int M, N; 
 int xOrigin, yOrigin; 
 int temp1, temp2;
 int rFirst, rLast;
 int rSize;
 if (nrhs < 2) 
{
 mexErrMsgTxt("Too few input arguments");
}
if (nrhs > 2)
{
mexErrMsgTxt("Too many input arguments");
}
if (nlhs > 2)
{
mexErrMsgTxt("Too many output arguments to RADON");
}
if (mxIsSparse(I) || mxIsSparse(THETA))
{
mexErrMsgTxt("Sparse inputs not supported");
}
if (!mxIsDouble(I) || !mxIsDouble(THETA))
{
mexErrMsgTxt("Inputs must be double");
}
deg2rad = 3.14159265358979 / 180.0;
numAngles = mxGetM(THETA) * mxGetN(THETA);
thetaPtr = (double *) mxCalloc(numAngles, sizeof(double));
pr1 = mxGetPr(THETA);
pr2 = thetaPtr;
for (k = 0; k < numAngles; k++)
*(pr2++) = *(pr1++) * deg2rad;
M = mxGetM(I);
N = mxGetN(I);
xOrigin = MAXX(0, (N-1)/2);
yOrigin = MAXX(0, (M-1)/2);
temp1 = M - 1 - yOrigin;
temp2 = N - 1 - xOrigin;
rLast = (int) ceil(sqrt((double) (temp1*temp1+temp2*temp2))) + 1;
rFirst = -rLast;
rSize = rLast - rFirst + 1;
if (nlhs == 2) 
{
R = mxCreateDoubleMatrix(rSize, 1, mxREAL);
pr1 = mxGetPr(R);
for (k = rFirst; k <= rLast; k++)
*(pr1++) = (double) k;
}
if (mxIsComplex(I))
{
 P = mxCreateDoubleMatrix(rSize, numAngles, mxCOMPLEX);
radon(mxGetPr(P), mxGetPr(I), thetaPtr, M, N, xOrigin, yOrigin,numAngles, rFirst, rSize);
radon(mxGetPi(P), mxGetPi(I), thetaPtr, M, N, xOrigin, yOrigin,numAngles, rFirst, rSize);
}
else
{
P = mxCreateDoubleMatrix(rSize, numAngles, mxREAL);
radon(mxGetPr(P), mxGetPr(I), thetaPtr, M, N, xOrigin, yOrigin,numAngles, rFirst, rSize);
}
}
static void radon(double *pPtr, double *iPtr, double *thetaPtr, int M, int N,int xOrigin, int yOrigin, int numAngles, int rFirst, int rSize)
{
int k, m, n, p;
double angle; 
double cosine, sine;
 double *pr; 
double *pixelPtr; 
double pixel; 
double rIdx; 
 int rLow; 
double pixelLow; 
double *yTable, *xTable; 
double *ySinTable, *xCosTable;
yTable = (double *) mxCalloc(2*M, sizeof(double));
xTable = (double *) mxCalloc(2*N, sizeof(double));
xCosTable = (double *) mxCalloc(2*N, sizeof(double));
ySinTable = (double *) mxCalloc(2*M, sizeof(double));
yTable[2*M-1] = -yOrigin - 0.25; 
for (k = 2*M-2; k >=0; k--)  
yTable[k] = yTable[k+1] + 0.5;
xTable[0] = -xOrigin - 0.25;
for (k = 1; k < 2*N; k++) 
xTable[k] = xTable[k-1] + 0.5;
for (k = 0; k < numAngles; k++)
{
angle = thetaPtr[k];
pr = pPtr + k*rSize;
cosine = cos(angle);
sine = sin(angle);
for (p = 0; p < 2*N; p++)
xCosTable[p] = xTable[p] * cosine - rFirst;
for (p = 0; p < 2*M; p++)
ySinTable[p] = yTable[p] * sine; 
for (n = 0; n < 2*N; n++)
{
pixelPtr = iPtr + (n/2)*M;
for (m = 0; m < 2*M; m++)
{
pixel = *pixelPtr;
if (pixel)
{
pixel *= 0.25; 
rIdx = (xCosTable[n] + ySinTable[m]);
rLow = (int) rIdx;
pixelLow = pixel*(1 - rIdx + rLow); 
pr[rLow++] += pixelLow; 
pr[rLow] += pixel - pixelLow; 
}
if (m%2)
pixelPtr++;
}
}
}
mxFree((void *) yTable);
mxFree((void *) xTable);
mxFree((void *) xCosTable);
mxFree((void *) ySinTable);
}
