//////////////////////////////////////////////////////////////////////////// 
// Image_Processing_toolbox_2
//
// Copyright (c) by S. Parua Biswas, CDAC, Kolkata.  2013
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function Retinex = retinex_McCann(L,nIter)
    //Computes the raw retinex output from an intensity image.
    
    [lhs,rhs]=argn(0);
    if rhs~=2 then
       error('There must be 2 aurguments. 1st one is for image and 2nd one is for the number of iterations.');
    end
    global OPE RRE Maximum
    [nrows ncols] = size(L);
    nLayers = ComputeLayers(nrows,ncols);
    nrows = nrows/(2^nLayers);
    ncols = ncols/(2^nLayers);
    if (nrows*ncols > 25) then
        error('The image size is invalid')
    end
    Maximum = max(L(:));
    
    OP = Maximum*ones(nrows, ncols);
    for layer = 0:nLayers
        RR = ImageDownResolution(L,2^(nLayers-layer));
 //       disp(RR)
        OPE = [zeros(nrows,1) OP zeros(nrows,1)];
        OPE = [zeros(1,ncols+2); OPE; zeros(1,ncols+2)];
        
        RRE = [RR(:,1) RR RR(:,$)];
        RRE = [RRE(1,:); RRE; RRE($,:)];
        for iter = 1:nIter
            CompareWithNeighbor(-1,0);
            CompareWithNeighbor(-1,1);
            CompareWithNeighbor(0,1);
            CompareWithNeighbor(1,1);
            CompareWithNeighbor(1,0);
            CompareWithNeighbor(1,-1);
            CompareWithNeighbor(0,-1);
            CompareWithNeighbor(-1,-1);
        end
        NP = OPE(2:($-1), 2:($-1));
        OP = NP(:, [fix(1:0.5:ncols) ncols]);
        OP = OP([fix(1:0.5:nrows) nrows], :);
        nrows = 2*nrows; 
        ncols = 2*ncols;       
    end
 
    Retinex = NP;
   endfunction 
    //////////////////////////////
    function CompareWithNeighbor(dif_row, dif_col)
        global OPE, RRE, Maximum

//        disp(Maximum)
 //       disp(OPE(2+dif_row:($-1+dif_row)))
//        disp(2+dif_col: ($-1+dif_col))
        IP = OPE(2+dif_row:($-1+dif_row), 2+dif_col: ($-1+dif_col))+ RRE(2:($-1),2:($-1)) - RRE(2+dif_row:($-1+dif_row), 2+dif_col:($-1+dif_col));
        
        IP(IP > Maximum) = Maximum; //The reset step
        
        
        if (dif_col == -1) then
            IP(:,1) = OPE(2:($-1),2);
        end
        if (dif_col == 1) then
            IP(:,$) = OPE(2:($-1),$-1);
        end
        if (dif_row == -1) then
            IP(1,:) = OPE(2, 2:($-1));
        end
        if (dif_row == 1) then
            IP($,:)=OPE($-1,2:($-1));
        end
        NP = (OPE(2:($-1),2:($-1))+IP)/2;
//        disp(NP)
        OPE(2:($-1),2:($-1))=NP;
        endfunction
        ///////////////////////////
    function Layers = ComputeLayers(nrows,ncols)
 
        power = 2^fix(log2(gcd_val(nrows,ncols)));
 //       disp(power)
        while(power > 1 & ((modulo(nrows,power)~=0) | (modulo(ncols,power) ~= 0))),
            power = power/2;
            disp(power)
        end
        Layers = log2(power);
    endfunction
        
        //////////////////////////////
        function Result = ImageDownResolution(A, blocksize)
            [rows, cols] = size(A);
            result_rows = rows/blocksize;
            result_cols = cols/blocksize;
 //           disp(result_rows)
 //           disp(result_cols)
            Result = zeros(result_rows, result_cols);
 //           disp(Result)
            for crt_row = 1:result_rows
                for crt_col = 1:result_cols
                    Result(crt_row, crt_col) = mean(A(1+(crt_row-1)*blocksize:crt_row*blocksize,...
                    1+(crt_col-1)*blocksize:crt_col*blocksize));
                end
            end
        endfunction  
   /////////////////////////
   function y=gcd_val(a,b)
    a=abs(a);
    b=abs(b);
    if(a<b)  then
        ii=a;
    else
        ii=b;
    end
    for i=1:ii
        if ((modulo(a,i)==0) & (modulo(b,i)==0)) then
            y=i;
        end
    end
endfunction 
